@@ -1,41 +1,31 @@
-#' A class that contains multiple calculation groups.
+
+#' R6 class that contains multiple calculation groups.
 #'
-#' The PivotCalculationGroups class stores all of the calculation groups for a
-#' pivot table.
+#' @description
+#' The `PivotCalculationGroups` class stores all of the
+#' calculation groups for a pivot table.  Every pivot table
+#' has at least one pivot calculation group and this is
+#' sufficient for all regular pivot tables.  Additional
+#' calculation groups are typically only created for
+#' irregular/custom pivot tables.
+#' See the "Irregular Layout" vignette for an example.
 #'
 #' @docType class
 #' @importFrom R6 R6Class
 #' @import jsonlite
-#' @return Object of \code{\link{R6Class}}.
 #' @format \code{\link{R6Class}} object.
 #' @examples
 #' # This class should only be created by the pivot table.
 #' # It is not intended to be created outside of the pivot table.
-#' @field parentPivot Owning pivot table.
-
-#' @section Methods:
-#' \describe{
-#'   \item{Documentation}{For more complete explanations and examples please see
-#'   the extensive vignettes supplied with this package.}
-#'   \item{\code{new(...)}}{Create a new container for pivot table calculation
-#'   groups, specifying the field values documented above.}
-#'
-#'   \item{\code{isExistingCalculationGroup(calculationGroupName)}}{Check if a
-#'   calculation group exists with the specified name.}
-#'   \item{\code{getCalculationGroup(calculationGroupName)}}{Get the calculation
-#'   group with the specified name.}
-#'   \item{\code{addCalculationGroup(calculationGroupName)}}{Create a new
-#'   calculation group with the specified name.}
-#'   \item{\code{asList()}}{Get a list representation of these calculation
-#'   groups.}
-#'   \item{\code{asJSON()}}{Get a JSON representation of these calculation
-#'   groups.}
-#'   \item{\code{asString()}}{Get a text representation of these calculation
-#'   groups.}
-#' }
 
 PivotCalculationGroups <- R6::R6Class("PivotCalculationGroups",
   public = list(
+
+    #' @description
+    #' Create a new `PivotCalculationGroups` object.
+    #' @param parentPivot The pivot table that this `PivotCalculationGroups`
+    #' instance belongs to.
+    #' @return A new `PivotCalculationGroups` object.
     initialize = function(parentPivot) {
       if(parentPivot$argumentCheckMode > 0) {
         checkArgument(parentPivot$argumentCheckMode, FALSE, "PivotCalculationGroups", "initialize", parentPivot, missing(parentPivot), allowMissing=FALSE, allowNull=FALSE, allowedClasses="PivotTable")
@@ -45,6 +35,11 @@ PivotCalculationGroups <- R6::R6Class("PivotCalculationGroups",
       private$p_groups <- list()
       if(private$p_parentPivot$traceEnabled==TRUE) private$p_parentPivot$trace("PivotCalculationGroups$new", "Created new Pivot Calculation Groups.")
     },
+
+    #' @description
+    #' Check if a calculation group exists with the specified name.
+    #' @param calculationGroupName The name of the calculation group.
+    #' @return `TRUE` if the calculation group already exists, `FALSE` otherwise.
     isExistingCalculationGroup = function(calculationGroupName=NULL) {
       if(private$p_parentPivot$argumentCheckMode > 0) {
         checkArgument(private$p_parentPivot$argumentCheckMode, FALSE, "PivotCalculationGroups", "isExistingCalculationGroup", calculationGroupName, missing(calculationGroupName), allowMissing=FALSE, allowNull=FALSE, allowedClasses="character")
@@ -55,7 +50,30 @@ PivotCalculationGroups <- R6::R6Class("PivotCalculationGroups",
       if(private$p_parentPivot$traceEnabled==TRUE) private$p_parentPivot$trace("PivotCalculationGroups$isExistingCalculationGroup", "Checked calculation group exists.")
       return(invisible(calcGroupExists))
     },
-    item = function(index) { return(invisible(private$p_groups[[index]])) },
+
+    #' @description
+    #' Retrieve a calculation group by index.
+    #' @param index An integer specifying the calculation group to retrieve.
+    #' @return The calculation group that exists at the specified index.
+    item = function(index) {
+      if(private$p_parentPivot$argumentCheckMode > 0) {
+        checkArgument(private$p_parentPivot$argumentCheckMode, FALSE, "PivotCalculationGroups", "item", index, missing(index), allowMissing=FALSE, allowNull=FALSE, allowedClasses=c("integer", "numeric"))
+      }
+      if(private$p_parentPivot$traceEnabled==TRUE) private$p_parentPivot$trace("PivotCalculationGroups$item", "Getting calculation group...")
+      if(index<1) {
+        stop(paste0("PivotCalculationGroups$index(): index must be greater than 0."), call. = FALSE)
+      }
+      if(index>length(private$p_groups)) {
+        stop(paste0("PivotCalculationGroups$index(): index must be less than or equal to ", length(private$p_groups), "."), call. = FALSE)
+      }
+      if(private$p_parentPivot$traceEnabled==TRUE) private$p_parentPivot$trace("PivotCalculationGroups$item", "Got calculation group....")
+      return(invisible(private$p_groups[[index]]))
+    },
+
+    #' @description
+    #' Retrieve a calculation group by name.
+    #' @param calculationGroupName The name of the calculation group to retrieve.
+    #' @return The calculation group with the specified name.
     getCalculationGroup = function(calculationGroupName=NULL) {
       if(private$p_parentPivot$argumentCheckMode > 0) {
         checkArgument(private$p_parentPivot$argumentCheckMode, FALSE, "PivotCalculationGroups", "getCalculationGroup", calculationGroupName, missing(calculationGroupName), allowMissing=FALSE, allowNull=FALSE, allowedClasses="character")
@@ -64,12 +82,17 @@ PivotCalculationGroups <- R6::R6Class("PivotCalculationGroups",
                                     list(calculationGroupName=calculationGroupName))
       calculationGroup <- private$p_groups[[calculationGroupName]]
       if(is.null(calculationGroup)) {
-        stop(paste0("PivotCalculationGroups$getCalculationGroup(): No calculationGroup exists with the name '",
+        stop(paste0("PivotCalculationGroups$getCalculationGroup(): No calculation group exists with the name '",
                     calculationGroupName, "'"), call. = FALSE)
       }
       if(private$p_parentPivot$traceEnabled==TRUE) private$p_parentPivot$trace("PivotCalculationGroups$getCalculationGroup", "Got calculation group.")
       return(invisible(calculationGroup))
     },
+
+    #' @description
+    #' Create a new calculation group.
+    #' @param calculationGroupName The name of the calculation group to create
+    #' @return The new calculation group.
     addCalculationGroup = function(calculationGroupName=NULL) {
       if(private$p_parentPivot$argumentCheckMode > 0) {
         checkArgument(private$p_parentPivot$argumentCheckMode, FALSE, "PivotCalculationGroups", "addCalculationGroup", calculationGroupName, missing(calculationGroupName), allowMissing=FALSE, allowNull=FALSE, allowedClasses="character")
@@ -86,6 +109,10 @@ PivotCalculationGroups <- R6::R6Class("PivotCalculationGroups",
       if(private$p_parentPivot$traceEnabled==TRUE) private$p_parentPivot$trace("PivotCalculationGroups$addCalculationGroup", "Added calculation group.")
       return(invisible(calculationGroup))
     },
+
+    #' @description
+    #' Return the contents of this object as a list for debugging.
+    #' @return A list of various object properties.
     asList = function() {
       lst <- list()
       if(length(private$p_groups) > 0) {
@@ -97,7 +124,17 @@ PivotCalculationGroups <- R6::R6Class("PivotCalculationGroups",
       }
       return(invisible(lst))
     },
+
+    #' @description
+    #' Return the contents of this object as JSON for debugging.
+    #' @return A JSON representation of various object properties.
     asJSON = function() { return(jsonlite::toJSON(self$asList())) },
+
+    #' @description
+    #' Return a representation of this object as a character value.
+    #' @param seperator A character value used when concatenating
+    #' the text representations of different calculation groups.
+    #' @return A character summary of various object properties.
     asString = function(seperator=", ") {
       if(private$p_parentPivot$argumentCheckMode > 0) {
         checkArgument(private$p_parentPivot$argumentCheckMode, FALSE, "PivotCalculationGroups", "asString", seperator, missing(seperator), allowMissing=TRUE, allowNull=FALSE, allowedClasses="character")
@@ -115,8 +152,13 @@ PivotCalculationGroups <- R6::R6Class("PivotCalculationGroups",
     }
   ),
   active = list(
+    #' @field count The number of calculation groups in the pivot table.
     count = function(value) { return(invisible(length(private$p_groups))) },
+
+    #' @field groups A list containing the calculation groups in the pivot table.
     groups = function(value) { return(invisible(private$p_groups)) },
+
+    #' @field defaultGroup The default calculation group in the pivot table.
     defaultGroup = function(value) { return(invisible(private$p_defaultGroup)) }
   ),
   private = list(
